%----------------------------------------------------------------
% Routine     : peplot.m (PE - Plot)
%
% Description : Plots the results from a PE-analysis in the 
%               'plotfig' window.               
%               
% Language    : Matlab V4.1
% Written by  : Vingmed sound A/S, Trond Kleveland 
%
% Version no. : 1.0 TKL 17.09.93 First version
%             : 1.1 TKL 20.09.93 Changed call to petxt.m
%             : 1.2 TKL 22.09.93 Removed analysis from call
%             : 1.3 TKL 22.11.93 Added Time - analysis,
%                                short filenames
%             : 1.4 TKL 24.11.93 Added yz-plane to analysis
%             : 1.5  SH 02.12.93 Added flag as input variable
%             : 1.6  SH 10.12.93 Added normalization factor for annulars
%             : 1.7 TKL 21.02.94 Added xy-plane to analysis
%             : 1.8 BJE 07.10.94 Extended movie, removed some view options
%             : 1.9 BJE 12.10.94 changed function parameters, added moviebreak
%             : 2.0 BJE 21.11.94 Added max_resp, min_resp for autoscale
%             : 2.1 BJE 01.12.94 Added possibility to save the movie-matrix
%                                PC-compatibility
%             : 2.2 BJE 23.04.94 Removed ConfigOrigHandle, from 
%                                function call, Colormap -> setcomap()
%             : 2.3 SH  10.05.96 Added subplot command to initialize graph window
%               2.4 SH  29.05.97 Matlab 5: removed statements that gave warnings
%                                (test for isempty after input statements)
%
%               2.5 KI  02.07.97 Matlab 4: gif animating option
%                   SH  02.07.97 computer-dependent fprintf
%               2.6 SH  23.01.98 initialized gif and display for Matlab ver
%               5
%               7.0 SH 09.02.05 '' to deal with spaces in filenames
%
% Called by   : usimcnf
% Calling     : cleanscr, petxt, clipl, setcomap
%----------------------------------------------------------------


function peplot(resp, max_resp, min_resp, excitation, transducer, media, ...
      observasjon, option, flagg, x, y, z, t, config, ...
      plotfig, plotfigOrigHandle, usr_resdir)
  
  
%
% make up a vector of analysis times 
%
  if option(1) == 'm'                   % movie requested
      delta_t=1/(1e3*excitation(14)*observasjon(15));
      TimeStepVector = [observasjon(4):delta_t:observasjon(8)];      
      TLen = length(TimeStepVector);
    else
      TLen = 1;
  end
  
%
% INITIALIZATION
%
  clc
  cleanscr(plotfig,plotfigOrigHandle);
  figure(plotfig)
  subplot(1, 1, 1)

  MeshCont = 'Mesh';
  setcomap(plotfig,5);		% set colormap to 'cool'
  
%
% normalization factor (number of elements)
%
  if (flagg(2) == 1),
      
%
% rectangular array
%
      n_elem = transducer(2)*transducer(6);
    else
      
%
% annular array
%
      n_elem = transducer(2);
  end   
%
%   determine computer-dependent path separator
%
  csys=computer;
  if (csys(1:2) == 'PC'),
      d = '\';
    else
      d = '/';
  end
  
%
% set plot options : envelope/rf, lin/log
%
  clc
  disp('Set the following plot options: ')
  disp('  ')
  
  plot_type = input('-> envelope / rf [e/r] ? ','s');
  if isempty(plot_type)| all(plot_type ~= 'r')
      plot_type = 'e';          % default
  end
  
  if all(plot_type == 'r')
      plot_option = 'lin';      % can not plot rf logarithmic
    else
      plot_option=input('-> linear / logarithmic [lin/log] ? ','s');
      if isempty(plot_option)|all(plot_option ~= 'log')
          plot_option = 'lin';  % default
      end
  end
  
%
% set view option
%
  disp(' ')
  disp(' ')
  disp('... and view options: ')
      disp(' ')
  disp('1 - 3D view default')
  disp('2 - 3D from above')
  disp(' ')
  plot_view=input('-> Select view [1,2] : ');
  if isempty(plot_view)|plot_view ~= 2
      plot_view = 1;	% default
  end
  gif_anim='n';
  vers = version;
  if vers(1)=='4'
    if (csys(1:2) ~= 'PC') & (csys(1:2)~='MA')
      gif_anim = input('-> make gif-animation [n/y] ? ','s');
      disp('  ')
    end
  else
%    disp('Cannot make gif-animation in Matlab version 5')
  end
  if isempty(gif_anim), gif_anim ='n'; end  
%
% ... view & vtxt
%
  if (plot_view == 1)			% 3D default
      tmp_view=3;
      vtxt = ['   View: 3D default']; 
    elseif (plot_view == 2)		% 3D from above
      tmp_view=[-20 85];
      vtxt = ['   View: 3D above'];
  end
  
%
% set the text variables
%
  [Xtxt,Ytxt,Ztxt,Ttxt1,Ttxt2,Ttxt3] = ...
      petxt(excitation,media,transducer,option,flagg,z,t, ...
      plot_type,plot_option,MeshCont);
  
%
% set axis, depending on distance or time analysis
%
  if option(1) == '2' | option(1) == 'm' 
      if all(option(2:3) == 'xz') 
          Xx = x(1,:)*1e3;
          Zz = z(:,1)'*1e3;
        elseif all(option(2:3) == 'xy')
          Xx = x(1,:)*1e3;
          Zz = y(:,1)'*1e3;
        elseif all(option(2:3) == 'yz')
          Xx = y(1,:)*1e3;
          Zz = z(:,1)'*1e3;
        elseif all(option(2:3) == 'xt')
          Xx = x(1,:)*1e3;
          Zz = t(:,1)'*1e6;
      end
  end
  
%
% get current axes position modify it (plot axes)
%
  figure(plotfig);
  handle1 = gca;
  axis('off');
  v1 = get(handle1,'Position');
  v1(4)=v1(4)-0.1;
  set(handle1,'Position',v1)
  
%  
% make another axes for text (text axes)
%
  v2    = v1;
  v2(1) = v2(1) - 0.1;
  v2(2) = v2(2) + v2(4) + 0.05;
  v2(4) = 0.1;
  handle2 = axes('Position',v2);
  axis('off');
  
%      
% add some text ...
%
  text(0 , 1.0 ,Ttxt1);
  text(0 , 0.5 ,Ttxt2);
  tmp = [Ttxt3,vtxt];
  text(0 , 0.0 ,tmp);      
  drawnow;
  
%          
% set max_z, min_z for later autoscale         
%          
  if (plot_type == 'e') & (plot_option == 'lin')
      max_z = norm(max_resp);
      min_z = 0;
    elseif (plot_type == 'e') & (plot_option == 'log') 
      max_z = 20*log10(norm(max_resp)/n_elem + eps);
      min_z = -50;
    elseif (plot_type == 'r') & (plot_option == 'lin') 
      max_z = max_resp(1);
      min_z = min_resp(1);
  end
  
%
% INITIALIZE MOVIE
%
  figure(plotfig);
  axes(handle1);
  if (TLen > 1)
      M = moviein(TLen,handle1);
  end
  

  
%
% loop over TLen
%
  for PNo = 1 : TLen  
      
%
% load resp from file for movie
%
      if option(1) == 'm'
          if (PNo < 10)
              eval(['load ''',usr_resdir,'''',d,'Pict00',num2str(PNo),'.mat'])
              eval(['resp = Pict00',num2str(PNo),';'])
            elseif ((PNo >= 10)&(PNo < 100))
              eval(['load ''',usr_resdir,'''',d,'Pict0',num2str(PNo),'.mat'])
              eval(['resp = Pict0',num2str(PNo),';'])
            elseif ((PNo >= 100)&(PNo < 1000))
              eval(['load ''',usr_resdir,'''',d,'Pict',num2str(PNo),'.mat'])
              eval(['resp = Pict',num2str(PNo),';'])
          end
      end
      
%          
% set resp according to plot options         
%          
      if (plot_type == 'e') & (plot_option == 'lin')
          resp = abs(resp);
        elseif (plot_type == 'e') & (plot_option == 'log') 
          resp = 20*log10(abs(resp)/n_elem + eps);
        elseif (plot_type == 'r') & (plot_option == 'lin') 
          resp = real(resp);
      end
      
% 
% plot it
%
      if (option(1) == 'm')
	if (csys(1:2) == 'PC') | (csys(1:2) == 'MA')
	  fprintf('plot number %3.0f of totally %3.0f plots\n',PNo,TLen);
	else
	  fprintf('plot number %3.0f of totally %3.0f plots\r',PNo,TLen);
	end
      end      
      if all( plot_option == 'log' )
          mesh(Zz, Xx, clipl(resp,-50))
        else
          mesh(Zz, Xx, resp)
      end


  

%          
% set axis and view
%
      tmp_axis = axis;
      tmp_axis(5) = min_z;
      tmp_axis(6) = max_z;
      axis(tmp_axis);
      view(tmp_view);
      
%      
% snap current plot for movie
%
      M(:,PNo) = getframe; 
%
% make gif files: gif.A1, gif.A2, ..., gif.B1, ...gif.Z9
%
  if isempty(gif_anim)| (gif_anim == 'n')
      gif_anim = 'n';          % default
  else
   ext=sprintf('%c%d',64+fix((10+PNo)/10),rem(PNo,10));
   filename = ['gif.',ext];
   printgif( '-dgif8', filename )
  end      
%  
% end of loop over TLen
%
  end
 
   if (gif_anim == 'y')
    eval(['!gpgconvert'])
   end 
%
% set labels
%      
  xlabel(Xtxt);
  ylabel(Ytxt);
  zlabel(Ztxt);                                         
  
%  
% play movie
%
  if (option(1) == 'm')
      more = 'y';     
      while (more == 'y')
%          clc
          disp('Movie options :')
          disp('  ')
          n=input('-> Enter number of movie loops [1..100] : ');
              if isempty(n)
              n = 10;   % default
          end 
          fps=input('-> Enter speed in frames/sec [1..20] : ');
              if isempty(fps)
              fps = 8;  % default
          end
%
% make up a vector that plays the movie with a break after each loop.
% To maintain speed, this is done with a vector N to specify the order
% of the frames.
%
          last_picture=size(M,2);
          N=[n, 1:last_picture , ones(1,fps)*last_picture]; % break for 1 sec 
          disp('  ');
          disp('Movie is playing ...');
              figure(plotfig);
          movie(M,N,fps);   
          disp('  ')
          more = input('-> Do you want more movie [y/n] ? ','s');
	  if isempty(more), more = 'y'; end
	end
%
% save movie matrix ?
%
%      if (input('-> Do you want to save the movie [y/n] ? ','s') == 'y')
%          [fname,path]=uiputfile('lastone.mvi','Save movie as ...');
%          if ~(fname(1) == 0)
%              eval(['save ',path,fname,' M N']);
%	      disp(' ');
%	      disp(['Saved movie as ',path,fname]);
%          end
%      end
      
  end
  
  fprintf('\n\nFinished!\n');
  
